@@ -1,5 +1,6 @@
 # imports # {{{ 
 from django.utils import simplejson
+from django.conf.urls.defaults import url
 from django.conf import settings
 from django import forms
 from django.core.files.base import ContentFile
@@ -749,14 +750,15 @@ class JSONEncoder(simplejson.JSONEncoder):
 
 # form_handler # {{{
 def form_handler(
-    request, form_cls, require_login=False, block_get=False,
-    next=None, template=None, login_url=None, 
+    request, form_cls, require_login=False, block_get=False, ajax=False,
+    next=None, template=None, login_url=None, pass_request=True
 ):
     """
     Some ajax heavy apps require a lot of views that are merely a wrapper
     around the form. This generic view can be used for them.
     """
     from django.shortcuts import render_to_response
+    is_ajax = require.is_ajax() or ajax or request.REQUEST.get("json") == "true"
     if login_url is None: 
         login_url = getattr(settings, "LOGIN_URL", "/login/")
     if callable(require_login): 
@@ -766,9 +768,10 @@ def form_handler(
     if require_login:
         if require_login == "404":
             raise Http404("login required")
-        return HttpResponseRedirect(
-            "%s?next=%s" % (login_url, request.path) # TODO: use right path
-        )
+        redirect_url = "%s?next=%s" % (login_url, request.path) # FIXME
+        if is_ajax:
+            return JSONResponse({ 'success': False, 'redirect': redirect_url })
+        return HttpResponseRedirect(redirect_url)
     if block_get and request.method != "POST":
         raise Http404("only post allowed")
     if isinstance(form_cls, basestring):
@@ -783,24 +786,33 @@ def form_handler(
             template, {"form": form_cls(request)}, # TODO: allow defaults from URL?
             context_instance=RequestContext(request)
         )
-    form = form_cls(request, request.REQUEST)
+    if pass_request:
+        form = form_cls(request, request.REQUEST)
+    else:
+        form = form_cls(request.REQUEST)
     if form.is_valid():
         saved = form.save()
-        if request.REQUEST.get("json") == "true":
-            return JSONResponse({ 'success': True, 'response': saved })
+        if is_ajax: return JSONResponse({ 'success': True, 'response': saved })
         if next: return HttpResponseRedirect(next)
         if template: return HttpResponseRedirect(saved)
         return JSONResponse({ 'success': True, 'response': saved })
-    if request.REQUEST.get("json") == "true":
+    if is_ajax:
         return JSONResponse({ 'success': False, 'errors': form.errors })
     if template:
         return render_to_response(
-            template, {"form": form}, 
-            context_instance=RequestContext(request)
+            template, {
+                "form": form_cls(request) if pass_request else form_cls()
+            }, context_instance=RequestContext(request)
         )
     return JSONResponse({ 'success': False, 'errors': form.errors })
 # }}}
 
+# fhurl # {{{ 
+def fhurl(reg, **kw):
+    name = kw.pop("name", None)
+    return url(reg, form_handler, kw, name=name)
+# }}} 
+
 # copy_file_to_s3 # {{{ 
 s3_operation_lock = threading.Condition(threading.Lock())
 def copy_file_to_s3(p, key, bucket):
@@ -827,10 +839,7 @@ def clean_data(func):
 # }}}
 
 # get address book from google # {{{
-
-
 class GContacts(object):
-    
     def __init__(self, email, password):
         import gdata.contacts
         self.gd_client = gdata.contacts.service.ContactsService()
@@ -983,7 +992,7 @@ def templated(template, mimetype="text/html"):
     through without any processing.
     """
     from django.shortcuts import render_to_response
-    from django.generic.views.simple import direct_to_template
+    from django.views.generic.simple import direct_to_template
     def decorator(view):
         @wraps(view)
         def wrapped(request, *args, **kwargs):
